﻿using System;
using System.Collections.Generic;
using Amplitude.AI.Common.DataStructure;
using Amplitude.AI.Common.IO.Contract.Attributes;
using Amplitude.AI.Core.Brain.Allocators;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.Population;
using Amplitude.AI.Core.Decision.Resource;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots.ContextData;
using Amplitude.AI.Core.Snapshots.ContextDataJob;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.GroundBattle;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.ShipDesign;
using Amplitude.AI.Core.Snapshots.Position;
using Newtonsoft.Json;

namespace Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony
{
    [SnapshotType("ColonizedStarSystem")]
    public class ColonizedStarSystemSnapshot : ResourceDataProvider, INodePositionnable, IHeroAssignable, IEntitySnapshot, ISnapshot
    {
        public ColonizedStarSystemSnapshot()
        {
            this.PendingProductions = new List<ConstructionSnapshot>();
            this.ShipDesignContextData = new List<ConstructibleContextData>();
            this.StarSystemImprovementContextData = new List<ConstructibleContextData>();
            this.PlanetColonizationContextData = new List<ConstructibleContextData>();
            this.OutpostActionContextData = new List<OutpostActionSnapshot>();
            this.oupostActionJob = new OutpostActionJob(this.OutpostActionContextData);
        }

        [ArrayMethod("GetSimulationDescriptors")]
        [ClassProperty("Name")]
        public string[] DescriptorNames { get; set; }

        public ColonizedStarSystemSnapshot.StarSystemState State
        {
            get
            {
                return (ColonizedStarSystemSnapshot.StarSystemState)this.ColonyState;
            }
        }

        public NodePosition NodePosition
        {
            get
            {
                if (this.starSystemNode != null)
                {
                    return this.starSystemNode.NodePosition;
                }
                return NodePosition.Invalid;
            }
        }

        [HeaderProperty("Spaceport")]
        public SpaceportSnapshot SpaceportSnapshot { get; set; }

        [HeaderProperty("Citadel")]
        public CitadelSnapshot Citadel { get; set; }

        [SimulationProperty("RequiredHackingProgress")]
        public float RequiredHackingProgress { get; set; }

        [SimulationProperty]
        public float HackingSpeedMultiplierForSelf { get; set; }

        [SimulationProperty]
        public float HackingSpeedMultiplierForOthers { get; set; }

        [ClassProperty]
        public bool IsInDecay { get; set; }

        [HeaderProperty]
        public EntityVisibility Visibility { get; set; }

        public HeroSnapshot AssignedHero { get; set; }

        public HangarSnapshot Hangar { get; set; }

        public override bool IsActive
        {
            get
            {
                return this.State == ColonizedStarSystemSnapshot.StarSystemState.Colony || this.State == ColonizedStarSystemSnapshot.StarSystemState.Ghost || this.State == ColonizedStarSystemSnapshot.StarSystemState.Outpost;
            }
        }

        public bool IsColonyOrGhost
        {
            get
            {
                return this.State == ColonizedStarSystemSnapshot.StarSystemState.Colony || this.State == ColonizedStarSystemSnapshot.StarSystemState.Ghost;
            }
        }

        public bool IsBeingBlockaded
        {
            get
            {
                return this.BlockadeOrbitingMilitaryPower > 0f;
            }
        }

        public bool IsBeingBesieged
        {
            get
            {
                return this.OrbitingSiegePower > 0f;
            }
        }

        [SimulationProperty]
        public float OrbitingSiegePower { get; set; }

        [SimulationProperty]
        public float BlockadeOrbitingMilitaryPower { get; set; }

        [SimulationProperty]
        public float NeighbourRadius { get; set; }

        public bool IsBeingInvaded
        {
            get
            {
                return this.GroundBattle != null;
            }
        }

        public GroundBattleSnapshot GroundBattle { get; set; }

        [SimulationProperty]
        public float NetSystemManpower { get; set; }

        [SimulationProperty]
        public float SystemManpowerStock { get; set; }

        [SimulationProperty]
        public float MaximumSystemManpower { get; set; }

        [SimulationProperty]
        public float GroundBattleDefenderManpowerLimit { get; set; }

        [ArrayMethod("GetPopulations")]
        public PopulationSnapshot[] Populations { get; private set; }

        [ClassMethod("GetNextMinorPopulationAffinityToTransform")]
        public string NextMinorPopulationAffinityToTransformName
        {
            private get
            {
                return this.NextMinorPopulationAffinityToTransform.ToString();
            }
            set
            {
                this.NextMinorPopulationAffinityToTransform = PopulationSnapshot.GetAffinityFromName(value);
            }
        }

        [JsonIgnore]
        public PopulationSnapshot.FactionAffinity NextMinorPopulationAffinityToTransform { get; set; }

        [ClassMethod("GetSystemBuyOutCost", Synchronization = SnapshotElementAttribute.SynchronizationFlag.OncePerTurn)]
        public float SystemBuyoutCostByInfluence { get; private set; }

        [ClassMethod("GetNextPopulationGrowthStock", Synchronization = SnapshotElementAttribute.SynchronizationFlag.OncePerTurn)]
        public float NextPopulationGrowthStock { get; private set; }

        [SimulationProperty]
        public float GroundBattleDefenderTroopsMaxHealthMultiplier { get; set; }

        [SimulationProperty]
        public float GroundBattleDefenderTroopsDamageMultiplier { get; set; }

        [HeaderMethod("GetTraitorSystemAction")]
        public EntityActionSnapshot TraitorSystemAction { get; set; }

        public float TurnBeforeNextPopulation { get; set; }

        public bool CanBuyoutByInfluence
        {
            get
            {
                return this.SystemBuyoutCostByInfluence >= 0f;
            }
        }

        public ColonizedPlanetSnapshot[] ColonizedPlanets { get; set; }

        public List<ConstructionSnapshot> PendingProductions { get; set; }

        public List<ConstructibleContextData> ShipDesignContextData { get; set; }

        public List<ConstructibleContextData> StarSystemImprovementContextData { get; set; }

        public List<ConstructibleContextData> PlanetColonizationContextData { get; set; }

        public List<OutpostActionSnapshot> OutpostActionContextData { get; set; }

        public List<StarSystemImprovementSnapshot> StarSystemImprovements { get; set; }

        public int NumberOfOutpostToProvideGrowthTo { get; set; }

        [JsonIgnore]
        public ColonizedStarSystemSnapshot[] OutpostMigrationDestinationSystems { get; set; }

        [JsonIgnore]
        public StarSystemNodeSnapshot StarSystemNode
        {
            get
            {
                return this.starSystemNode;
            }
            set
            {
                if (this.starSystemNode != value)
                {
                    if (this.starSystemNode != null)
                    {
                        this.starSystemNode.Colonies.Remove(this);
                    }
                    this.starSystemNode = value;
                    if (this.starSystemNode != null)
                    {
                        this.starSystemNode.Colonies.Add(this);
                    }
                }
            }
        }

        [JsonIgnore]
        public ColonizedStarSystemSnapshot OutpostMigrationSource { get; private set; }

        private bool AllVisible { get; set; }

        public bool VisibleByAtLeastOne { get; set; }

        public bool VisibleByAtLeastOneMajor { get; set; }

        public bool HasTemples
        {
            get
            {
                return this.NumberOfTemples > 0f;
            }
        }

        public MothershipSnapshot AttachedMothership { get; private set; }

        public int TraitorCount { get; set; }

        public int[] TraitorCountByEmpire { get; set; }

        [ClassProperty("AntiVisionStrength")]
        public float InvisibilityStrength { get; set; }

        [ClassProperty("VisionStrength")]
        public float DetectionStrength { get; set; }

        public bool IsVisible(int empireIndex)
        {
            return this.AllVisible || this.Visibility.VisibilityFor(empireIndex) >= EntityVisibility.Layer.Known;
        }

        public float GetCitadelPower()
        {
            float num = 0f;
            if (this.Citadel == null || !this.Citadel.CanFireInSpaceBattle)
            {
                return num;
            }
            for (int i = 0; i < this.Citadel.Modules.Length; i++)
            {
                num += this.Citadel.Modules[i].OffensiveMilitaryPower;
            }
            for (int j = 0; j < this.StarSystemImprovements.Count; j++)
            {
                if (this.StarSystemImprovements[j].UniqueName == "StarSystemImprovementCitadelSpaceBattle2")
                {
                    num *= 1.5f;
                    break;
                }
            }
            return num;
        }

        public override void SynchronizeContextData()
        {
            base.SynchronizeContextData();
            if (base.EmpireOwner is MajorEmpireSnapshot)
            {
                if (this.State == ColonizedStarSystemSnapshot.StarSystemState.Colony)
                {
                    base.ResetConstructibleCost<ConstructibleContextData>(this.ShipDesignContextData, "ship-design", true, true, null, null);
                    base.ResetConstructibleCost<ConstructibleContextData>(this.StarSystemImprovementContextData, "star-system-improvement", true, true, null, null);
                    base.ResetConstructibleCost<ConstructibleContextData>(this.PlanetColonizationContextData, "planet-colonization", true, true, null, null);
                }
                if (this.State == ColonizedStarSystemSnapshot.StarSystemState.Outpost)
                {
                    this.ResetOutpostActionContextDataList();
                    return;
                }
            }
            else
            {
                this.ShipDesignContextData.Clear();
                this.StarSystemImprovementContextData.Clear();
                this.PlanetColonizationContextData.Clear();
                this.OutpostActionContextData.Clear();
            }
        }

        protected override void OnSynchronizationEnd()
        {
            base.OnSynchronizationEnd();
            if (this.TraitorCountByEmpire == null)
            {
                ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
                this.TraitorCountByEmpire = new int[service.GameSnapshot.EmpireSnapshotGuids.Length];
            }
            else
            {
                for (int i = 0; i < this.TraitorCountByEmpire.Length; i++)
                {
                    this.TraitorCountByEmpire[i] = 0;
                }
            }
            this.TraitorCount = 0;
        }

        public override void AfterSynchronize()
        {
            base.AfterSynchronize();
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            if (this.EmpireIndex >= 0)
            {
                if (base.EmpireOwner == null || base.EmpireOwner.EmpireIndex != this.EmpireIndex)
                {
                    this.ChangeOwner(service.GameSnapshot.EmpireSnapshots[this.EmpireIndex]);
                }
            }
            else if (base.EmpireOwner != null)
            {
                this.ChangeOwner(null);
            }
            this.AllVisible = true;
            this.VisibleByAtLeastOne = false;
            this.VisibleByAtLeastOneMajor = false;
            ISnapshotRepositoryService service2 = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            for (int i = 0; i < service2.GameSnapshot.EmpireSnapshots.Length; i++)
            {
                MajorEmpireSnapshot majorEmpireSnapshot = service2.GameSnapshot.EmpireSnapshots[i] as MajorEmpireSnapshot;
                if (this.Visibility.VisibilityFor(i) < EntityVisibility.Layer.Known)
                {
                    this.AllVisible = false;
                }
                if (i != this.EmpireIndex && this.Visibility.VisibilityFor(i) >= EntityVisibility.Layer.Known)
                {
                    this.VisibleByAtLeastOne = true;
                    if (majorEmpireSnapshot != null)
                    {
                        this.VisibleByAtLeastOneMajor = true;
                    }
                }
            }
            this.ColonizedPlanets = base.LoadEntitiesFromGuid<ColonizedPlanetSnapshot>(this.ColonizedPlanetGuids, this.ColonizedPlanets);
            for (int j = 0; j < this.ColonizedPlanets.Length; j++)
            {
                this.ColonizedPlanets[j].ColonizedStarSystem = this;
            }
            this.PendingProductions = base.LoadEntitiesFromGuid<ConstructionSnapshot>(this.PendingConstructionGuids, this.PendingProductions);
            StarSystemNodeSnapshot starSystemNodeSnapshot = null;
            if (service.TryGetEntitySnapshot<StarSystemNodeSnapshot>(this.StarSystemNodeGuid, out starSystemNodeSnapshot))
            {
                this.StarSystemNode = starSystemNodeSnapshot;
            }
            this.StarSystemImprovements = base.LoadEntitiesFromGuid<StarSystemImprovementSnapshot>(this.StarSystemImprovementGuids, this.StarSystemImprovements);
            this.OutpostMigrationSource = base.LoadEntityFromGuid<ColonizedStarSystemSnapshot>(this.OutpostMigrationSourceGuid);
            this.OutpostMigrationDestinationSystems = base.LoadEntitiesFromGuid<ColonizedStarSystemSnapshot>(this.OutpostMigrationDestinationSystemGuids, this.OutpostMigrationDestinationSystems);
            this.NumberOfOutpostToProvideGrowthTo = this.OutpostMigrationDestinationSystems.Length;
            Array.Sort<PopulationSnapshot>(this.Populations, (PopulationSnapshot left, PopulationSnapshot right) => left.Count.CompareTo(right.Count));
            if (this.SpaceportSnapshot != null)
            {
                this.SpaceportSnapshot.AfterSynchronize();
            }
            if (this.SystemGrowthNet <= 0f)
            {
                this.TurnBeforeNextPopulation = float.MaxValue;
            }
            else
            {
                this.TurnBeforeNextPopulation = (this.NextPopulationGrowthStock - this.SystemGrowthStock) / this.SystemGrowthNet;
            }
            this.GroundBattle = null;
            List<EntitySnapshot> entitySnapshots = service.GetEntitySnapshots(typeof(GroundBattleSnapshot));
            for (int k = 0; k < entitySnapshots.Count; k++)
            {
                GroundBattleSnapshot groundBattleSnapshot = entitySnapshots[k] as GroundBattleSnapshot;
                if (groundBattleSnapshot.DefenderColonizedStarSystemGUID == base.GameEntityGuid)
                {
                    this.GroundBattle = groundBattleSnapshot;
                }
            }
            this.ComputeManpowerByEmpire();
            this.AttachedMothership = null;
            if (base.EmpireOwner.HasMotherships && this.StarSystemNode.DockedFleets != null)
            {
                for (int l = 0; l < this.StarSystemNode.DockedFleets.Length; l++)
                {
                    FleetSnapshot fleetSnapshot = this.StarSystemNode.DockedFleets[l];
                    if (fleetSnapshot.EmpireIndex == this.EmpireIndex && fleetSnapshot.HasMothershipAttached)
                    {
                        for (int m = 0; m < fleetSnapshot.Ships.Count; m++)
                        {
                            MothershipSnapshot mothershipSnapshot = fleetSnapshot.Ships[m] as MothershipSnapshot;
                            if (mothershipSnapshot != null && mothershipSnapshot.IsAttached)
                            {
                                this.AttachedMothership = mothershipSnapshot;
                                return;
                            }
                        }
                        return;
                    }
                }
            }
        }

        public override void Release()
        {
            base.Release();
            this.StarSystemNode = null;
            this.EmpireIndex = -1;
            this.ChangeOwner(null);
            this.ShipDesignContextData.Clear();
            this.StarSystemImprovementContextData.Clear();
            this.PlanetColonizationContextData.Clear();
            this.OutpostActionContextData.Clear();
        }

        public override void ChangeOwner(EmpireSnapshot empireSnapshot)
        {
            if (base.EmpireOwner != null)
            {
                base.EmpireOwner.ColonizedStarSystems.Remove(this);
            }
            base.ChangeOwner(empireSnapshot);
            if (base.EmpireOwner != null)
            {
                base.EmpireOwner.ColonizedStarSystems.Add(this);
            }
        }

        public ConstructibleContextData GetConstructibleData(string elementKey)
        {
            for (int i = 0; i < this.StarSystemImprovementContextData.Count; i++)
            {
                ConstructibleContextData constructibleContextData = this.StarSystemImprovementContextData[i];
                if (elementKey == constructibleContextData.ElementKey)
                {
                    return constructibleContextData;
                }
            }
            for (int j = 0; j < this.PlanetColonizationContextData.Count; j++)
            {
                ConstructibleContextData constructibleContextData2 = this.PlanetColonizationContextData[j];
                if (elementKey == constructibleContextData2.ElementKey)
                {
                    return constructibleContextData2;
                }
            }
            for (int k = 0; k < this.ShipDesignContextData.Count; k++)
            {
                ConstructibleContextData constructibleContextData3 = this.ShipDesignContextData[k];
                if (elementKey == constructibleContextData3.ElementKey)
                {
                    return constructibleContextData3;
                }
            }
            return null;
        }

        public override bool NeedSync()
        {
            return true;
        }

        public override bool NeedContextDataSync()
        {
            if (base.EmpireOwner != null && base.EmpireOwner.ShipDesignSnapshots != null)
            {
                for (int i = 0; i < base.EmpireOwner.ShipDesignSnapshots.Count; i++)
                {
                    ShipDesignSnapshot shipDesignSnapshot = base.EmpireOwner.ShipDesignSnapshots[i];
                    if (!shipDesignSnapshot.Hidden)
                    {
                        ConstructibleContextData constructibleContextData = null;
                        for (int j = 0; j < this.ShipDesignContextData.Count; j++)
                        {
                            ConstructibleContextData constructibleContextData2 = this.ShipDesignContextData[j];
                            if (constructibleContextData2.ElementKey == shipDesignSnapshot.Name)
                            {
                                constructibleContextData = constructibleContextData2;
                                break;
                            }
                        }
                        if (constructibleContextData == null)
                        {
                            return true;
                        }
                    }
                }
            }
            return base.NeedContextDataSync();
        }

        public PopulationSnapshot GetPopulationByAffinity(PopulationSnapshot.FactionAffinity populationAffinity)
        {
            for (int i = 0; i < this.Populations.Length; i++)
            {
                if (this.Populations[i].Affinity == populationAffinity)
                {
                    return this.Populations[i];
                }
            }
            return null;
        }

        public bool HasPendingProduction(string productionName)
        {
            for (int i = 0; i < this.PendingProductions.Count; i++)
            {
                if (this.PendingProductions[i].ElementName == productionName)
                {
                    return true;
                }
            }
            return false;
        }

        protected override string GeneratePath()
        {
            return base.GeneratePath();
        }

        protected override void FillResourceAllocatorList(List<ResourceAllocator> resourceAllocators)
        {
            resourceAllocators.Add(new ResourceAllocatorIndustry(this));
            resourceAllocators.Add(new ResourceAllocator_Population(this));
            resourceAllocators.Add(new GrowthAllocator(this));
        }

        private void ResetOutpostActionContextDataList()
        {
            this.oupostActionJob.InitializeMessage(base.GameEntityGuid);
            base.StartVolatileTask(this.oupostActionJob);
        }

        private void ComputeManpowerByEmpire()
        {
            if (this.manpowerByEmpire == null)
            {
                ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
                this.manpowerByEmpire = new float[service.GameSnapshot.EmpireSnapshots.Length];
            }
            for (int i = 0; i < this.manpowerByEmpire.Length; i++)
            {
                this.manpowerByEmpire[i] = 0f;
                if (i == base.EmpireOwner.EmpireIndex)
                {
                    this.manpowerByEmpire[i] += this.SystemManpowerStock;
                    if (this.Citadel != null)
                    {
                        this.manpowerByEmpire[i] += this.Citadel.CitadelManpowerStock;
                    }
                }
                EmpireSnapshot empireSnapshot;
                if (this.GroundBattle != null && Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>().TryGetEntitySnapshot<EmpireSnapshot>(this.GroundBattle.AttackerEmpireGuid, out empireSnapshot) && empireSnapshot.EmpireIndex == i)
                {
                    this.manpowerByEmpire[i] += this.GroundBattle.AttackerTotalManPower;
                }
                StarSystemNodeSnapshot starSystemNodeSnapshot = this.StarSystemNode;
                if (starSystemNodeSnapshot.DockedFleets != null)
                {
                    for (int j = 0; j < starSystemNodeSnapshot.DockedFleets.Length; j++)
                    {
                        FleetSnapshot fleetSnapshot = starSystemNodeSnapshot.DockedFleets[j];
                        if (fleetSnapshot.EmpireIndex == i)
                        {
                            this.manpowerByEmpire[i] += fleetSnapshot.ShipManpowerStock;
                        }
                    }
                }
            }
        }

        [ClassField("empireIndex", Synchronization = SnapshotElementAttribute.SynchronizationFlag.OnlyOnce)]
        public int EmpireIndex { get; set; }

        [ClassProperty(Synchronization = SnapshotElementAttribute.SynchronizationFlag.OnlyOnce)]
        public string LocalizedName { get; set; }

        [ArrayProperty("PlanetsColonized")]
        [ClassProperty("GUID")]
        public ulong[] ColonizedPlanetGuids { get; set; }

        [ArrayMethod("GetPendingConstructions", Parameter = "SystemProduction")]
        [ClassProperty("GUID")]
        public ulong[] PendingConstructionGuids { get; set; }

        [SimulationProperty("Population")]
        public float Population { get; set; }

        [SimulationProperty("MaximumPopulation")]
        public float MaximumPopulation { get; set; }

        [ClassProperty("SupervisorName")]
        public string Supervisor { get; set; }

        [SimulationProperty("NetSystemGrowth")]
        public float SystemGrowthNet { get; set; }

        [SimulationProperty("SystemGrowthStock")]
        public float SystemGrowthStock { get; set; }

        [SimulationProperty("NetSystemProduction")]
        public float NetSystemProduction { get; set; }

        [ClassMethod]
        public int PirateFleetCountForPirateMark { get; set; }

        public float SystemProductionNet
        {
            get
            {
                if (this.NetSystemProduction == 0f)
                {
                    return this.SystemProduction - this.SystemProductionFromProductionConversion - this.SystemProductionUpkeep;
                }
                return this.NetSystemProduction - this.SystemProductionFromProductionConversion;
            }
        }

        [SimulationProperty("SystemProduction")]
        public float SystemProduction { get; set; }

        [SimulationProperty("SystemProductionUpkeep")]
        public float SystemProductionUpkeep { get; set; }

        [SimulationProperty("SystemProductionStock")]
        public float SystemProductionStock { get; set; }

        [SimulationProperty("NetSystemMoney")]
        public float SystemMoneyNet { get; set; }

        [SimulationProperty("NetSystemEmpirePoint")]
        public float SystemPrestigeNet { get; set; }

        [SimulationProperty("NumberOfTemples")]
        public float NumberOfTemples { get; set; }

        [ClassProperty("State")]
        public int ColonyState { get; set; }

        [HeaderProperty("Node")]
        [ClassProperty("GUID")]
        public ulong StarSystemNodeGuid { get; set; }

        [ArrayProperty("StarSystemImprovements")]
        [ClassProperty("GUID")]
        public ulong[] StarSystemImprovementGuids { get; set; }

        [ClassMethod("HasPortal")]
        public bool HasPortal { get; set; }

        [ClassProperty]
        public float Level { get; set; }

        public bool IsHomeSystem
        {
            get
            {
                return this.StarSystemNode.HomeSystemEmpireIndex != -1;
            }
        }

        public bool IsBeingLeeched
        {
            get
            {
                return this.LeechingEmpireIndex != -1;
            }
        }

        [SimulationProperty]
        public float Happiness { get; set; }

        [ClassProperty("LeechingEmpireIndex")]
        public int LeechingEmpireIndex { get; set; }

        [SimulationProperty]
        public float Ownership { get; set; }

        [ClassProperty]
        public float TurnsBeforeAutomatedShipRequest { get; set; }

        [ClassProperty("BaseTurnsBeforeAutomatedShipRequest")]
        public float MaxTurnsBeforeAutomatedShipRequest { get; set; }

        [ClassProperty]
        public bool IsSuitableForOutpostMigration { get; set; }

        [ArrayProperty("RebellionSuppressionActions")]
        [ClassMethod("GetCost")]
        public float[] RebellionSuppressionActionsCosts { get; set; }

        [ArrayProperty("RebellionSuppressionActions")]
        [ClassProperty("ResourceName")]
        public string[] RebellionSuppressionActionsResources { get; set; }

        [ArrayProperty("RebellionSuppressionActions")]
        [ClassProperty("Name")]
        public string[] RebellionSuppressionActionsActionNames { get; set; }

        [ClassProperty]
        public float RebellionValue { get; set; }

        [ClassMethod("GetRebellionTriggerValue", Synchronization = SnapshotElementAttribute.SynchronizationFlag.OncePerTurn)]
        public float RebellionTriggerValue { get; set; }

        [ClassMethod("GetRebellionProgressPerTurn", Synchronization = SnapshotElementAttribute.SynchronizationFlag.OncePerTurn)]
        public float RebellionProgressPerTurn { get; set; }

        [HeaderProperty("OutpostMigrationSource")]
        [ClassProperty("GUID")]
        private ulong OutpostMigrationSourceGuid { get; set; }

        [ArrayProperty("OutpostMigrationDestinationSystems")]
        [ClassProperty("GUID")]
        private ulong[] OutpostMigrationDestinationSystemGuids { get; set; }

        static ColonizedStarSystemSnapshot()
        {
        }

        [SimulationProperty("SystemProductionFromProductionConversion")]
        public float SystemProductionFromProductionConversion { get; set; }

        public static readonly string SystemSupervisorBalanced = "SystemSupervisorAIBalancedPopulation";

        public float[] manpowerByEmpire;

        private StarSystemNodeSnapshot starSystemNode;

        private OutpostActionJob oupostActionJob;

        public enum StarSystemState
        {
            None,
            Colony,
            Outpost,
            Lost,
            Ghost
        }
    }
}
