﻿using System;
using Amplitude.AI.Common.DataStructure;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.ContextData;
using Amplitude.AI.Core.Snapshots.DatabaseSnapshots;
using Amplitude.AI.Core.Snapshots.DatabaseSnapshots.IndustryImprovements;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;

namespace Amplitude.AI.Core.Brain.Analysis.Jobs.EmpireBased.ResourceNeeds
{
    public class ComputeStrategicTargetStock : AnalysisJob
    {
        protected override void Compute()
        {
            MajorEmpireSnapshot majorEmpireSnapshot = base.EmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            IDatabase<ScienceConstructibleSnapshot> technologyDatabase;
            if (!Amplitude.AI.Core.Framework.Services.GetService<IDatabaseRepositoryService>().TryGetDatabase<ScienceConstructibleSnapshot>(out technologyDatabase))
            {
                return;
            }
            AnalysisDataHolder analysisDataHolder = base.EmpireSnapshot.AIPlayer.AnalysisDataHolder;
            for (int i = 0; i < analysisDataHolder.MarketplaceData.StrategicData.Length; i++)
            {
                MarketplaceData.ResourceData resourceData = analysisDataHolder.MarketplaceData.StrategicData[i];
                HeuristicFloat targetStock = default(HeuristicFloat);
                targetStock.Add(5f, "Always try to keep a constant minimum amount in stock");
                HeuristicFloat operand = this.CalculateStockForImprovements(resourceData.ResourceName, majorEmpireSnapshot, analysisDataHolder);
                targetStock.Add(operand, "Amount of this strategic resource to keep spare for improvements");
                HeuristicFloat operand2 = this.CalculateStockForModules(resourceData.ResourceName, majorEmpireSnapshot, technologyDatabase);
                targetStock.Add(operand2, "Amount of this strategic resource to keep spare for modules");
                HeuristicFloat operand3 = this.CalculateStockForHulls(resourceData.ResourceName, majorEmpireSnapshot);
                targetStock.Add(operand3, "Amount of this strategic resource to keep spare for hulls");
                float operand4 = 999f;
                targetStock.Min(operand4, "We should never aim for more than the maximum possible stock");
                resourceData.TargetStock = targetStock;
            }
        }

        private HeuristicFloat CalculateStockForImprovements(string resourceName, MajorEmpireSnapshot majorEmpire, AnalysisDataHolder analysisDataHolder)
        {
            HeuristicFloat result = new HeuristicFloat(0f, "");
            float num = 0f;
            for (int i = 0; i < majorEmpire.ColonizedStarSystems.Count; i++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot = majorEmpire.ColonizedStarSystems[i];
                if (colonizedStarSystemSnapshot.State == ColonizedStarSystemSnapshot.StarSystemState.Colony)
                {
                    num += 1f;
                    NodeData analysisData = colonizedStarSystemSnapshot.StarSystemNode.GetAnalysisData<NodeData>(base.EmpireIndex);
                    int count = analysisData.Constructibles.Count;
                    for (int j = 0; j < count; j++)
                    {
                        Amplitude.AI.Core.Snapshots.DatabaseSnapshots.IndustryImprovements.StarSystemImprovementSnapshot starSystemImprovementSnapshot = analysisData.Constructibles[j].ConstructibleSnapshot as Amplitude.AI.Core.Snapshots.DatabaseSnapshots.IndustryImprovements.StarSystemImprovementSnapshot;
                        if (starSystemImprovementSnapshot != null && (!starSystemImprovementSnapshot.HasUniqueTags || i <= 0))
                        {
                            ConstructibleContextData constructibleContextData = analysisData.Constructibles[j].ConstructibleContextData;
                            CostSnapshot[] costs = constructibleContextData.CostsSnapshot.Costs;
                            if (costs != null)
                            {
                                for (int k = 0; k < costs.Length; k++)
                                {
                                    if (costs[k].ResourceName == resourceName)
                                    {
                                        result.Add(costs[k].Amount, "Improvement '{0}' on System '{1}' needs this resource", constructibleContextData.ElementKey, colonizedStarSystemSnapshot.LocalizedName);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if (num != 0f)
            {
                result.Divide(num, "colonyCount");
            }
            return result;
        }

        private HeuristicFloat CalculateStockForModules(string resourceName, MajorEmpireSnapshot majorEmpire, IDatabase<ScienceConstructibleSnapshot> technologyDatabase)
        {
            HeuristicFloat result = default(HeuristicFloat);
            for (int i = 0; i < majorEmpire.TechnologyContextData.Count; i++)
            {
                TechnologyContextData technologyContextData = majorEmpire.TechnologyContextData[i];
                ScienceConstructibleSnapshot scienceConstructibleSnapshot;
                if (technologyContextData.State == TechnologyContextData.TechnologyState.Researched && technologyDatabase.TryGetValue(technologyContextData.ElementKey, out scienceConstructibleSnapshot) && scienceConstructibleSnapshot.GainsSnapshot.HasGain("ModuleGain", resourceName))
                {
                    result.Add(10f, "The technology '{0}' unlocks one or more modules which use this resource", scienceConstructibleSnapshot.LocalizeName);
                }
            }
            return result;
        }

        private HeuristicFloat CalculateStockForHulls(string resourceName, MajorEmpireSnapshot majorEmpire)
        {
            HeuristicFloat result = new HeuristicFloat(0f, "");
            for (int i = 0; i < majorEmpire.AvailableHulls.Count; i++)
            {
                HullDefinitionSnapshot hullDefinitionSnapshot = majorEmpire.AvailableHulls[i];
                for (int j = 0; j < hullDefinitionSnapshot.ResourceNeeded.Length; j++)
                {
                    if (hullDefinitionSnapshot.ResourceNeeded[j] == resourceName)
                    {
                        result.Add(20f, "Hull '{0}' needs this resource", hullDefinitionSnapshot.Name);
                    }
                }
            }
            return result;
        }

        public ComputeStrategicTargetStock()
        {
        }

        private const float MinimumStockToMaintain = 5f;
    }
}
