﻿using System;
using Amplitude.AI.Core.AIPlayers.MajorEmpire;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Brain.Goals.Diplomacy;
using Amplitude.AI.Core.Brain.Strategic.Data.Diplomacy;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Modules;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Diplomacy;

namespace Amplitude.AI.Core.Brain.Generators.Diplomacy.Behavior
{
    public abstract class ProposeTerm : DefaultContextBasedGenerator<MajorDiplomacyRelationData, Win, WantMakeDiplomaticContract>
    {
        public override string LocalizedName
        {
            get
            {
                return string.Format("{0} of {1}", this.PropositionMethod, this.WantedTermReference);
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 10;
            }
        }

        public abstract string WantedTermReference { get; }

        public virtual bool SkipBalanceContract
        {
            get
            {
                return false;
            }
        }

        public abstract DiplomaticContractSnapshot.PropositionMethod PropositionMethod { get; }

        protected abstract MajorDiplomacyRelationData.DiplomaticContractDesire DiplomaticDesire { get; }

        protected abstract void PreFillContract(AIPlayer aiPlayer, MajorDiplomacyRelationData context, WantMakeDiplomaticContract goal);

        protected override void PrepareDecision(AIPlayer aiPlayer, MajorDiplomacyRelationData context)
        {
            base.PrepareDecision(aiPlayer, context);
            WantMakeDiplomaticContract wantMakeDiplomaticContract = aiPlayer.Blackboard.FindFirstRunning<WantMakeDiplomaticContract>((WantMakeDiplomaticContract x) => x.WantedTermReference == this.WantedTermReference && x.PropositionMethod == this.PropositionMethod && x.OtherEmpireIndex == context.OtherEmpireSnapshot.EmpireIndex);
            if (wantMakeDiplomaticContract == null)
            {
                return;
            }
            wantMakeDiplomaticContract.Clear();
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            IDecisionNetworkService service = Amplitude.AI.Core.Framework.Services.GetService<IDecisionNetworkService>();
            ITimeService service2 = Amplitude.AI.Core.Framework.Services.GetService<ITimeService>();
            ISnapshotRepositoryService service3 = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            AIPlayer_MajorEmpire aiplayer_MajorEmpire = aiPlayer as AIPlayer_MajorEmpire;
            if (aiplayer_MajorEmpire != null)
            {
                for (int i = 0; i < aiplayer_MajorEmpire.StrategicDataHolder.DiplomacyData.RelationDataByEmpireIndex.Length; i++)
                {
                    if (!service3.GameSnapshot.EmpireSnapshots[i].HasBeenEliminated)
                    {
                        MajorDiplomacyRelationData majorDiplomacyRelationData = aiplayer_MajorEmpire.StrategicDataHolder.DiplomacyData.RelationDataByEmpireIndex[i] as MajorDiplomacyRelationData;
                        if (majorDiplomacyRelationData != null && (majorDiplomacyRelationData.LastContractTryDate != service2.CurrentTurn || majorDiplomacyRelationData.LastContractTryDecisionCount == service.GetDecisionCount(aiPlayer.ControlledEmpireIndex)) && (this.PropositionMethod != DiplomaticContractSnapshot.PropositionMethod.Negotiation || service2.CurrentTurn - majorDiplomacyRelationData.LastNegotiationPropositionDate >= 2) && this.DiplomaticDesire == majorDiplomacyRelationData.ChosenDiplomaticDesire)
                        {
                            base.ContinueDecision(aiPlayer, majorDiplomacyRelationData);
                        }
                    }
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, MajorDiplomacyRelationData context)
        {
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<Win>(VictorySnapshot.VictoryType.Score));
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, MajorDiplomacyRelationData context, Win parent)
        {
            WantMakeDiplomaticContract wantMakeDiplomaticContract = aiPlayer.Blackboard.FindFirstRunningWantMakeDiplomaticContract(context.OtherEmpireSnapshot.EmpireIndex, this.WantedTermReference, this.PropositionMethod);
            if (wantMakeDiplomaticContract == null)
            {
                wantMakeDiplomaticContract = new WantMakeDiplomaticContract(aiPlayer.Blackboard.GenerateGUID());
                wantMakeDiplomaticContract.WantedTermReference = this.WantedTermReference;
                wantMakeDiplomaticContract.PropositionMethod = this.PropositionMethod;
                wantMakeDiplomaticContract.OtherEmpireIndex = context.OtherEmpireSnapshot.EmpireIndex;
                wantMakeDiplomaticContract.EvaluationOffset = context.ChosenDiplomaticDesireEvaluation;
                wantMakeDiplomaticContract.SkipBalanceContract = this.SkipBalanceContract;
                aiPlayer.Blackboard.Register<WantMakeDiplomaticContract>(wantMakeDiplomaticContract);
            }
            if (wantMakeDiplomaticContract.BalanceContractStatus == BalanceDiplomaticContract.DiplomaticContractStatus.None)
            {
                this.PreFillContract(aiPlayer, context, wantMakeDiplomaticContract);
            }
            Diagnostics.Log("Want make contract {0} from {1} to {2} at decision {3}", new object[]
			{
				wantMakeDiplomaticContract.WantedTermReference,
				aiPlayer.ControlledEmpireSnapshot.EmpireIndex,
				context.OtherEmpireSnapshot.EmpireIndex,
				context.LastContractTryDecisionCount
			});
            base.ContinueDecision(aiPlayer, base.GetGenerationData(context, parent, wantMakeDiplomaticContract));
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<MajorDiplomacyRelationData, Win, WantMakeDiplomaticContract> generationData)
        {
            return new HeuristicFloat(1f, "");
        }

        protected ProposeTerm()
        {
        }

        private const int MinimumNumberOfTurnsBetweenPropositions = 2;
    }
}
